%define debug_package %{nil}

%define mybuildnumber %{?build_number}%{?!build_number:1}

%global selinuxvariants targeted
%global modulename      ebpf_exporter
%global toolchain       clang

Name:           ebpf_exporter
Version:        2.2.0.2
Release:        %{mybuildnumber}%{?dist}
Summary:        Prometheus exporter for custom eBPF metrics

License:        GPLv3+
URL:            https://github.com/Rudd-O/ebpf_exporter
Source0:	https://github.com/Rudd-O/%{name}/archive/{%version}.tar.gz#/%{name}-%{version}.tar.gz

BuildRequires:  make
BuildRequires:  clang
BuildRequires:  coreutils
BuildRequires:  golang-src >= 1.11
BuildRequires:  golang-bin >= 1.11
BuildRequires:  tar
BuildRequires:  gawk
BuildRequires:  sed
BuildRequires:  findutils
BuildRequires:  bcc-devel
BuildRequires:  libbpf-devel
BuildRequires:  systemd-rpm-macros
%{?systemd_requires}
BuildRequires: selinux-policy-devel
%{?selinux_requires}

%description
Prometheus exporter for custom eBPF metrics.

Motivation of this exporter is to allow you to write eBPF code and export
metrics that are not otherwise accessible from the Linux kernel.

eBPF was described by Ingo Molnár as:

    One of the more interesting features in this cycle is the ability
    to attach eBPF programs (user-defined, sandboxed bytecode executed
    by the kernel) to kprobes. This allows user-defined instrumentation
    on a live kernel image that can never crash, hang or interfere with
    the kernel negatively.

An easy way of thinking about this exporter is bcc tools as prometheus metrics.

%prep
%setup -q
%if 0%{?fedora} <= 29
>&2 echo this is not buildable in {?fedora} because bcc-devel is too old.
false
%endif

%build
make build-binary GO_BUILD_FLAGS="-mod=vendor"

# Reset to GCC.
unset FFLAGS FCFLAGS CFLAGS CXXFLAGS CXX CC LDFLAGS
%global toolchain       gcc

for selinuxvariant in %selinuxvariants; do
    make NAME=${selinuxvariant} -f %{_datadir}/selinux/devel/Makefile
    bzip2 -9 %{modulename}.pp
    mv %{modulename}.pp.bz2 %{modulename}.pp.bz2.${selinuxvariant}
    make NAME=${selinuxvariant} -f %{_datadir}/selinux/devel/Makefile clean
done

# Remove unsupported CLANG flags.
export CFLAGS=$(echo "$CFLAGS" | sed 's/-mtune=generic //')
export CFLAGS=$(echo "$CFLAGS" | sed 's/-fstack-clash-protection //')
export CFLAGS=$(echo "$CFLAGS" | sed 's/-fstack-protector-strong //')
export CFLAGS=$(echo "$CFLAGS" | sed -r 's/-fcf-protection([^ ]* |$)//')

cd examples
rm -f cgroup* kfree_skb.* percpu-softirq.* syscalls.* tcp-window-clamps.* timers.* udp-drops.*
make build
cd ..

%install
mkdir -p $RPM_BUILD_ROOT/%{_bindir}
cp %{name} $RPM_BUILD_ROOT/%{_bindir}
mkdir -p $RPM_BUILD_ROOT/%{_unitdir}
cp *.service $RPM_BUILD_ROOT/%{_unitdir}
mkdir -p $RPM_BUILD_ROOT/%{_sysconfdir}/default
echo 'EBPF_EXPORTER_OPTS="--config.dir %{_libexecdir}/%{name} --config.names biolatency"' > $RPM_BUILD_ROOT/%{_sysconfdir}/default/%{name}
mkdir -p $RPM_BUILD_ROOT/%{_libexecdir}/%{name}/
cp examples/*.yaml examples/*.bpf.o $RPM_BUILD_ROOT/%{_libexecdir}/%{name}/

for selinuxvariant in %selinuxvariants; do
    install -d %{buildroot}%{_datadir}/selinux/${selinuxvariant}
    install -p -m 644 %{modulename}.pp.bz2.${selinuxvariant} \
           %{buildroot}%{_datadir}/selinux/${selinuxvariant}/%{modulename}.pp.bz2
done

%pre
for selinuxvariant in %selinuxvariants; do
  %selinux_relabel_pre -s $selinuxvariant
done

%post
for selinuxvariant in %selinuxvariants; do
  %selinux_modules_install -s $selinuxvariant %{_datadir}/selinux/${selinuxvariant}/%{modulename}.pp.bz2
done
%if 0%{?el6} || 0%{?el5}
chkconfig --add %{name}
%else
%systemd_post %{name}.service
%endif

%preun
%if 0%{?el6} || 0%{?el5}
if [ $1 -eq 0 ] ; then
    service %{name} stop > /dev/null 2>&1
    chkconfig --del %{name}
fi
%else
%systemd_preun %{name}.service
%endif

%postun
for selinuxvariant in %selinuxvariants; do
  %selinux_modules_uninstall -s $selinuxvariant %{modulename}
done
%if 0%{?el6} || 0%{?el5}
if [ "$1" -ge "1" ] ; then
    service %{name} condrestart >/dev/null 2>&1 || :
fi
%else
%systemd_postun %{name}.service
%endif

%posttrans
for selinuxvariant in %selinuxvariants; do
  %selinux_relabel_post -s $selinuxvariant
done


%files
%attr(0755, root, root) %{_bindir}/%{name}
%{_unitdir}/%{name}.service
%{_libexecdir}/%{name}/*.bpf.o
%{_libexecdir}/%{name}/*.yaml
%config(noreplace) %{_sysconfdir}/default/%{name}
%{_datadir}/selinux/*/%{modulename}.pp.bz2
%doc README.md examples/*

%changelog
* Tue Mar 26 2019 Manuel Amador (Rudd-O) <rudd-o@rudd-o.com>
- Initial release
